#include <language/modules/LinearSolverModule.hpp>

#include <algebra/LinearSolver.hpp>
#include <language/utils/BuiltinFunctionEmbedder.hpp>
#include <language/utils/TypeDescriptor.hpp>

LinearSolverModule::LinearSolverModule()
{
  this->_addBuiltinFunction("setLSVerbosity", std::make_shared<BuiltinFunctionEmbedder<void(const bool&)>>(

                                                [](const bool& verbose) -> void {
                                                  LinearSolverOptions::default_options.verbose() = verbose;
                                                }

                                                ));

  this->_addBuiltinFunction("setLSEpsilon", std::make_shared<BuiltinFunctionEmbedder<void(const double&)>>(

                                              [](const double& epsilon) -> void {
                                                LinearSolverOptions::default_options.epsilon() = epsilon;
                                              }

                                              ));

  this->_addBuiltinFunction("setLSMaxIter", std::make_shared<BuiltinFunctionEmbedder<void(const uint64_t&)>>(

                                              [](const uint64_t& max_iter) -> void {
                                                LinearSolverOptions::default_options.maximumIteration() = max_iter;
                                              }

                                              ));

  this->_addBuiltinFunction("setLSLibrary", std::make_shared<BuiltinFunctionEmbedder<void(const std::string&)>>(

                                              [](const std::string& library_name) -> void {
                                                LinearSolverOptions::default_options.library() =
                                                  getLSEnumFromName<LSLibrary>(library_name);
                                              }

                                              ));

  this->_addBuiltinFunction("setLSMethod", std::make_shared<BuiltinFunctionEmbedder<void(const std::string&)>>(

                                             [](const std::string& method_name) -> void {
                                               LinearSolverOptions::default_options.method() =
                                                 getLSEnumFromName<LSMethod>(method_name);
                                             }

                                             ));

  this->_addBuiltinFunction("setLSPrecond", std::make_shared<BuiltinFunctionEmbedder<void(const std::string&)>>(

                                              [](const std::string& precond_name) -> void {
                                                LinearSolverOptions::default_options.precond() =
                                                  getLSEnumFromName<LSPrecond>(precond_name);
                                              }

                                              ));

  this->_addBuiltinFunction("printLSOptions", std::make_shared<BuiltinFunctionEmbedder<void()>>(

                                                []() -> void {
                                                  std::cout << rang::fgB::yellow << "Linear solver options"
                                                            << rang::style::reset << '\n';
                                                  std::cout << LinearSolverOptions::default_options;
                                                }

                                                ));

  this->_addBuiltinFunction("printLSAvailable",
                            std::make_shared<BuiltinFunctionEmbedder<void()>>(

                              []() -> void {
                                std::cout << rang::fgB::yellow << "Available linear solver options"
                                          << rang::style::reset << '\n';
                                std::cout << rang::fgB::blue << " libraries" << rang::style::reset << '\n';
                                printLSEnumListNames<LSLibrary>(std::cout);
                                std::cout << rang::fgB::blue << " methods" << rang::style::reset << '\n';
                                printLSEnumListNames<LSMethod>(std::cout);
                                std::cout << rang::fgB::blue << " preconditioners" << rang::style::reset << '\n';
                                printLSEnumListNames<LSPrecond>(std::cout);
                              }

                              ));
}
